@@ -14,7 +14,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_resolver.c,v 1.2.2.3 2011/02/28 01:19:59 tbox Exp $ */
+/* $Id: t_resolver.c,v 1.3 2011/02/03 12:18:11 tbox Exp $ */
 
 #include <config.h>
 
@@ -85,7 +85,7 @@ make_resolver(dns_resolver_t **resolverp) {
 	isc_result_t result;
 
 	result = dns_resolver_create(view,
-			    task_manager, 1,
+			    task_manager, 1, 1,
 			    socket_manager,
 			    timer_manager,
 			    0, /* unsigned int options, */
